package com.sksamuel.scoot.rest.jersey.cache

import com.sun.jersey.spi.container._
import com.google.common.collect.Lists
import com.sun.jersey.api.model.AbstractMethod

import net.sf.ehcache.Cache
import net.sf.ehcache.CacheManager
import net.sf.ehcache.Ehcache
import net.sf.ehcache.Element
import net.sf.ehcache.config.CacheConfiguration
import net.sf.ehcache.config.MemoryUnit

import com.sun.jersey.spi.container.ContainerRequest
import com.sun.jersey.spi.container.ContainerRequestFilter
import com.sun.jersey.spi.container.ContainerResponse
import com.sun.jersey.spi.container.ContainerResponseFilter
import com.sun.jersey.spi.container.ResourceFilter
import java.util.Collections
import javax.ws.rs.core.Response
import java.util

/** @author Stephen Samuel */
class CachedResourceFilterFactory extends ResourceFilterFactory {

    override def create(am: AbstractMethod) = {
        val annotation = am.getAnnotation(classOf[CachedResource])
        if (annotation == null) {
            Collections.emptyList()
        } else {
            val filter = new CachedResourceFilter(annotation.cacheName, annotation.timeToLiveSeconds())
            val filters = new util.ArrayList[ResourceFilter]
            filters.add(filter)
            filters
        }
    }
}

class CachedResourceFilter(cacheName: String, timeToLiveSeconds: Int) extends ResourceFilter {

    val cacheConfig = new CacheConfiguration(cacheName, 0)
      .maxBytesLocalHeap(50, MemoryUnit.MEGABYTES)
      .timeToLiveSeconds(timeToLiveSeconds)
      .eternal(false)
      .timeToIdleSeconds(1)
    val manager = CacheManager.newInstance()
    val cache = manager.addCacheIfAbsent(new Cache(cacheConfig))

    override def getRequestFilter = {
        new ContainerRequestFilter() {

            override def filter(request: ContainerRequest) = {
                val key = request.getRequestUri.toString
                val element = cache.get(key)
                if (element == null || element.isExpired) {
                    request // no caching == no pre-processing
                } else {
                    val resp = element.getObjectValue.asInstanceOf[Response]
                    throw new CachedResourceException(resp)
                }
            }
        }
    }

    override def getResponseFilter = {
        new ContainerResponseFilter() {

            override def filter(request: ContainerRequest, response: ContainerResponse) = {
                val key = request.getRequestUri.toString
                val r =
                    Response.status(response.getStatus).entity(response.getEntity).`type`(response.getMediaType).build()
                cache.put(new Element(key, r))
                response
            }
        }
    }
}